//$Id: logreader.c 358 2010-07-01 20:10:57Z jakimmel $
// vim:foldmethod=marker:foldenable

/*
 * Front end for the audit library logreader.
 * Reads logs generated by audit library, accepts command line input with
 * format:
 * <name of this program> <flags starting with '-'> [--] <log files to read>
 *
 * Accepted flags:
 *	-m  : turn on displaying of non standard messages logs (default)
 *	-M  : turn off displaying of non standard messages logs
 *	-l  : turn on displaying standard entries in logs (default)
 *	-L  : turn off displaying standard entries in logs
 *	-o  : turn on displaying standard entries in logs (default)
 *	-O  : turn on displaying standard entries in logs (default)
 *	-h  : print in human readable form (default)
 *	-p  : print in parsable form
 *	-f [filename] : write output to given file. WARNING if file already
 *			exists contents will be lost
 *
 * Note: flags "-t", "-h" and "-p" override each other, so only one can be set
 * at a time. If more than one are specified in the command line, the last one
 * will take effect.
 *
 * Mustafa Paksoy
 * July 05
 */


//INCLUDE
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include "logreader.h"

//GLOBAL VARS
extern char *optarg;
extern int optind, opterr, optopt;
int retval = 0;

//LOCAL FUNC PROTOS
int main(int, char **);

//MAIN
/*
 * Main function takes command line arguments in format:
 * <name of this program> <flags starting with '-'> [--] <log files to read>
 * parses flags with function getopt()
 *
 */
int main(int argc, char *argv[]) {
    int i;
    unsigned long count = 0;
    aud_entry_t *curent;
    FILE *newout;
    FILE *logfile; //file pointer to logfile specefied in path

    //Initialize getopt with legitimate options
    int curopt, optcount;
    char *aud_optstr = "mMlLhpoOf:?";

    //set default option values
    optcount = 0;
    opterr = 0;
    options_t option = DEFOPT;

    //read next option
    while (-1 != (curopt =  GETOPT(aud_optstr))) {
	switch ((char) curopt) {
	//Message printing
	case 'm':
	    option.printmsg = 1;
	    break;
	case 'M':
	    option.printmsg = 0;
	    break;

	//Log entry printing
	case 'l':
	    option.printlog = 1;
	    break;
	case 'L':
	    option.printlog = 0;
	    break;

	//Print contents of log entries
	case 'o':
	    option.printtoken = 1;
	    break;
	case 'O':
	    option.printtoken = 0;
	    break;

	case 'h':

	case 'p':
	    option.format = (char) curopt;
	    break;

	//Set output file
	case 'f':
	    if ((newout = fopen(optarg, "w"))) {
		option.output = newout;
		printf("Successfully: opened file to write output: %s\n", optarg);
	    }
	    else {
		printf("Failed opening file to write output: %s\n", optarg);
	    }
	    break;

	case '?':
	    fprintf(stderr, "Invalid option flag: %c\n", (char) optopt);
            usage();
	    break;
	case ':':
	    fprintf(stderr, "Missing argument for: %s\n", argv[optind-1]);
	    break;
	default:
	    fprintf(stderr, "unexpected getopt return!!!\n");
            usage();
	    exit(-1);
	    break;
	}
	optcount++;
    }
#ifdef LOGREAD_DEBUG 
    printf("read %d options\n", optcount);
#endif 

    printf("Done reading options. %d options read.\n", optcount);
    for (i=optind; i<argc; i++) {
	printf("Reading log from file: %s\n",argv[i]);
	//read log
	if((logfile = audlib_get_logfile(argv[i]))){
	    printf("Successfully opened file %s\n", argv[i]);

	    printf("Output Preferences\nRegular entries:");
	    if (option.printlog) {
		printf("YES ");
	    }
	    else {
		printf("NO ");
	    }

	    printf("Notification entries:");
	    if (option.printmsg) {
		printf("YES ");
	    }
	    else {
		printf("NO ");
	    }

	    printf("Entry contents:");
	    if (option.printtoken) {
		printf("YES\n");
	    }
	    else {
		printf("NO\n");
	    }

	    printf("Output format: ");
	    if (option.format == 'h') {
		printf("human readable\n");
	    }
	    else if (option.format == 'p') {
		printf("parsable\n");
	    }
	    //display all contents
	    while((curent = audlib_get_entry(logfile))) {
		audlib_print_entry(curent,&option);
		free(curent);
		count++;
	    }
	    printf("Done displaying entries. %lu entries displayed.\n",count);
	    fclose(logfile);
	}
	else
	    printf("Reading log from file %s failed.\n", argv[i]);
    }

    puts("done.");
    return (retval);
}

void usage() {
fprintf(stderr,"%s\n","   Front end for the audit library logreader.");
fprintf(stderr,"%s\n","   Reads logs generated by audit library, accepts command line input with");
fprintf(stderr,"%s\n","   format:");
fprintf(stderr,"%s\n","   <name of this program> <flags starting with '-'> [--] <log files to read>");
fprintf(stderr,"%s\n","  ");
fprintf(stderr,"%s\n","   Accepted flags:");
fprintf(stderr,"%s\n","  	-m  : turn on displaying of non standard messages logs (default)");
fprintf(stderr,"%s\n","  	-M  : turn off displaying of non standard messages logs");
fprintf(stderr,"%s\n","  	-l  : turn on displaying standard entries in logs (default)");
fprintf(stderr,"%s\n","  	-L  : turn off displaying standard entries in logs");
fprintf(stderr,"%s\n","  	-o  : turn on displaying standard entries in logs (default)");
fprintf(stderr,"%s\n","  	-O  : turn on displaying standard entries in logs (default)");
fprintf(stderr,"%s\n","  	-h  : print in human readable form (default)");
fprintf(stderr,"%s\n","  	-p  : print in parsable form");
fprintf(stderr,"%s\n","  	-f [filename] : write output to given file. WARNING if file already");
fprintf(stderr,"%s\n","  			exists contents will be lost");
fprintf(stderr,"%s\n","  ");
fprintf(stderr,"%s\n","   Note: flags \"-t\", \"-h\" and \"-p\" override each other, so only one can be set");
fprintf(stderr,"%s\n","   at a time. If more than one are specified in the command line, the last one");
fprintf(stderr,"%s\n","   will take effect.");
fprintf(stderr,"%s\n","  ");
}
